//------------------------------------------------------------------------------
// CERN BE-CO-HT
// Mock Turtle
// https://gitlab.cern.ch/coht/mockturtle
//------------------------------------------------------------------------------
//
// unit name:   main
//
// description: A SystemVerilog testbench for the supplied SPEC MT demo.
//
//------------------------------------------------------------------------------
// Copyright CERN 2018
//------------------------------------------------------------------------------
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-2.0.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//------------------------------------------------------------------------------

`include "mock_turtle_driver.svh"
`include "vhd_wishbone_master.svh"

`timescale 1ns/1ps

module main;

   reg clk_125m = 0;

   always #4ns clk_125m <= ~clk_125m;

   // the Device Under Test
   spec_mt_demo #
     (
      .g_sim_bypass_pcie(1)
      )
   DUT
     (
      .clk_125m_pllref_p_i (clk_125m),
      .clk_125m_pllref_n_i (~clk_125m),
      .fp_led_green_o      (),
      .fp_led_red_o        (),
      .brd_button_i        (),
      .brd_led_o           (),
      .sim_wb_i            (Host.out),
      .sim_wb_o            (Host.in),
      .gn_rst_n_i          (),
      .gn_p2l_clk_n_i      (),
      .gn_p2l_clk_p_i      (),
      .gn_p2l_rdy_o        (),
      .gn_p2l_dframe_i     (),
      .gn_p2l_valid_i      (),
      .gn_p2l_data_i       (),
      .gn_p_wr_req_i       (),
      .gn_p_wr_rdy_o       (),
      .gn_rx_error_o       (),
      .gn_l2p_clkn_o       (),
      .gn_l2p_clkp_o       (),
      .gn_l2p_dframe_o     (),
      .gn_l2p_valid_o      (),
      .gn_l2p_edb_o        (),
      .gn_l2p_data_o       (),
      .gn_l2p_rdy_i        (),
      .gn_l_wr_rdy_i       (),
      .gn_p_rd_d_rdy_i     (),
      .gn_tx_error_i       (),
      .gn_vc_rdy_i         (),
      .gn_gpio_b           ());

   IVHDWishboneMaster Host
     (
      .clk_i   (DUT.clk_sys),
      .rst_n_i (DUT.rst_sys_n));

   IMockTurtleIRQ IrqMonitor (`MT_ATTACH_IRQ(DUT.U_Mock_Turtle));

   string fw = "../../../demos/hello_world/firmware/fw-01/fw-hello.bin";

   const uint64_t mt_base = 'h2_0000;

   MockTurtleDriver drv;

   initial begin

      @(posedge DUT.rst_sys_n);
      @(posedge DUT.clk_sys);

      drv = new (Host.get_accessor(), mt_base, IrqMonitor);

      drv.init();

      drv.enable_console_irq (0, 1);

      drv.load_firmware(0, fw, 1'b0);

      drv.reset_core(0, 0);

      forever begin
	 drv.update ();
	 #1us;
      end

   end // initial begin

endmodule // main
